#pragma once

#include <mcolor.h>
#include "token.hpp"
#include "Color4.hpp"
#include "Colors.hpp"
#include <source_location>
#include <map>
#include <jlog/token.hpp>
#include <Event.h>
#include "formatter.hpp"

namespace jlog
{
    using namespace mcolor;

    class LogEntry {
    public:
        LogEntry(Color4 cc, const std::string c, const std::string msg, const std::source_location t, const Timestamp ts);
    public:
        Color4 ContextColor() { return context_color; }
        std::string Context() { return context; }
        std::string Message() { return message; }
        std::source_location Trace() { return trace; }
        jlog::Timestamp GetTimeStamp() { return timestamp; };
        virtual std::vector<token> Tokenize();
    public:
        bool IncludeTrace;
        bool IncludeTimestamp;
    private:
        Color4 context_color;
        std::string context;
        std::string message;
        std::source_location trace;
        Timestamp timestamp;
    };

    class Logger {
    public:
        explicit Logger(const std::string& context, const Color4& color = Colors::LightGray);
    public:
        Event<LogEntry> OnLogEvent;
    public:
        void operator () (const std::string& message, const std::source_location& location = std::source_location::current());
        virtual void Log(const std::string &message, const std::source_location& location = std::source_location::current());
    public:
        void SetEnabled(bool state);
        void Enable();
        void Disable();
        bool Enabled();
        void LogFile(const std::string& f);
        std::string LogFile();
        // no cc no bullshit
        void SetColorCode(Color4 cc = Colors::LightGray);
        Color4 GetColorCode();
    public:
        std::string Context();
        void SetTraceback(bool b);
        void IncludeTimestamp(bool b);
        /**/
    protected:
        bool enabled = true;
        std::string logfile = "latest.log";
        Color4 colorcode = Colors::LightGray;
        std::string context;
        bool trace = true;
        bool timestamp = true;
    };
}