#include <jlog/logger.hpp>
#include <jlog/jlog.hpp>
#include <jlog/formatter.hpp>
#include <jlog/token.hpp>
#include "jlog/io.hpp"

namespace jlog
{
    LogEntry::LogEntry(Color4 cc, const std::string c, const std::string msg, const std::source_location t, const Timestamp ts) {
        this->context_color = cc;
        this->context = c;
        this->message = msg;
        this->trace = t;
        this->timestamp = ts;
    };

    std::vector<token> LogEntry::Tokenize() {
        std::vector<token> tokens;

        if (IncludeTimestamp)
            tokens.push_back({.colorCode = Colors::Gray, .content = std::format(
                    "{}-{}-{} {}:{}:{}.{}",
                    timestamp.Year(),
                    timestamp.Month(),
                    timestamp.Day(),
                    timestamp.Hour().count(),
                    timestamp.Minute().count(),
                    timestamp.Second().count(),
                    timestamp.Millisecond().count())});

        tokens.push_back({.colorCode = context_color, .content = context});

        if (IncludeTrace) {
            tokens.push_back({.colorCode = Colors::Gray, .content = trace.function_name()});
            tokens.push_back({.colorCode = Colors::Gray, .content = std::format("{}:{}",trace.file_name(), trace.line())});
        }

        tokens.push_back({.colorCode = Colors::Gray, .content = ">>", .delimiter = ""});

        tokens.push_back({.colorCode = Colors::Gray, .content = message, .delimiter = ""});

        return tokens;
    }

    Logger::Logger(const std::string& context, const Color4& color) {
        this->context = context;
        this->colorcode = color;
    }

    void Logger::Log(const std::string &message, const std::source_location& location) {
        if (!enabled)
            return;

        LogEntry logentry = {colorcode, context, message, location, Timestamp()};
        logentry.IncludeTrace = this->trace;
        logentry.IncludeTimestamp = this->timestamp;

        OnLogEvent(logentry);

        LogToConsole(logentry.Tokenize());
        LogToConsole("\n");
        LogToFile(logfile, logentry.Tokenize());
        LogToFile(logfile, "\n");
    }

    void Logger::SetEnabled(bool state) {
        this->enabled = state;
    }
    bool Logger::Enabled() { return this->enabled; }

    void Logger::LogFile(const std::string& f) {
        this->logfile = f;
    }
    std::string Logger::LogFile() { return this->logfile; }

    void Logger::SetColorCode(Color4 cc) { this->colorcode = cc; }
    Color4 Logger::GetColorCode() { return this->colorcode; }

    std::string Logger::Context() { return this->context; }

    void Logger::operator()(const std::string &message, const std::source_location& location) {
        Log(message, location);
    }

    void Logger::SetTraceback(bool b) {
        trace = b;
    }

    void Logger::Enable() { SetEnabled(true);}

    void Logger::Disable() { SetEnabled(false); }
}